@@ -1,6 +1,6 @@
 package JIRA::REST;
 # ABSTRACT: Thin wrapper around JIRA's REST API
-$JIRA::REST::VERSION = '0.015';
+$JIRA::REST::VERSION = '0.017';
 use 5.008_008;
 use utf8;
 use strict;
@@ -14,68 +14,92 @@ use JSON;
 use REST::Client;
 
 sub new {
-    my ($class, $URL, $username, $password, $rest_client_config) = @_;
+    my $class = shift; # this always has to come first!
 
-    # Make sure $URL isa URI
-    if (! defined $URL) {
-        croak __PACKAGE__ . "::new: URL argument must be defined.\n";
-    } elsif (! ref $URL) {
-        $URL = URI->new($URL);
-    } elsif (! $URL->isa('URI')) {
-        croak __PACKAGE__ . "::new: URL argument must be an URI object.\n";
+    # Valid option names in the order expected by the old-form constructor
+    my @opts = qw/url username password rest_client_config proxy ssl_verify_none anonymous/;
+
+    my %args;
+
+    if (@_ == 1 && ref $_[0] && ref $_[0] eq 'HASH') {
+        # The new-form constructor expects a single hash reference.
+        @args{@opts} = delete @{$_[0]}{@opts};
+        croak __PACKAGE__ . "::new: unknown arguments: '", join("', '", sort keys %{$_[0]}), "'.\n"
+            if keys %{$_[0]};
+    } else {
+        # The old-form constructor expects a list of positional parameters.
+        @args{@opts} = @_;
     }
 
-    my $path = $URL->path('');
-    # See if the user wants a specific JIRA Core REST API version:
-    if ($path =~ m@^/?$@) {
-        $path = '/rest/api/latest';
-    } elsif ($path !~ m@^/rest/api/(?:latest|\d+)$@) {
-        croak __PACKAGE__ . "::new: invalid path in URL: '$path'\n";
+    # Turn the url into a URI object
+    if (! $args{url}) {
+        croak __PACKAGE__ . "::new: 'url' argument must be defined.\n";
+    } elsif (! ref $args{url}) {
+        $args{url} = URI->new($args{url});
+    } elsif (! $args{url}->isa('URI')) {
+        croak __PACKAGE__ . "::new: 'url' argument must be a URI object.\n";
     }
 
-    # If username and password are not set we try to lookup the credentials
-    if (! defined $username || ! defined $password) {
-        ($username, $password) = _search_for_credentials($URL, $username);
+    my ($path, $api) = ($args{url}->path, '/rest/api/latest');
+    # See if the user wants a default REST API:
+    if ($path =~ s:(/rest/.*)$::) {
+        $api = $1;
+        $args{url}->path($path);
+    }
+    unless ($args{anonymous}) {
+        # If username and password are not set we try to lookup the credentials
+        if (! defined $args{username} || ! defined $args{password}) {
+            ($args{username}, $args{password}) =
+                _search_for_credentials($args{url}, $args{username});
+        }
+
+        foreach (qw/username password/) {
+            croak __PACKAGE__ . "::new: '$_' argument must be a non-empty string.\n"
+                unless defined $args{$_} && ! ref $args{$_} && length $args{$_};
+        }
     }
 
-    croak __PACKAGE__ . "::new: USERNAME argument must be a string.\n"
-        unless defined $username && ! ref $username && length $username;
-
-    croak __PACKAGE__ . "::new: PASSWORD argument must be a string.\n"
-        unless defined $password && ! ref $password && length $password;
-
-    $rest_client_config = {} unless defined $rest_client_config;
-    croak __PACKAGE__ . "::new: REST_CLIENT_CONFIG argument must be a hash-ref.\n"
-        unless
-        defined $rest_client_config
-        &&  ref $rest_client_config
-        &&  ref $rest_client_config eq 'HASH';
+    for ($args{rest_client_config}) {
+        $_ = {} unless defined;
+        croak __PACKAGE__ . "::new: 'rest_client_config' argument must be a hash reference.\n"
+            unless defined && ref && ref eq 'HASH';
+    }
 
     # remove the REST::Client faux config 'proxy' if set and use it later.
-    my $proxy = delete $rest_client_config->{proxy};
+    # This is deprecated since v0.017
+    if (my $proxy = delete $args{rest_client_config}{proxy}) {
+        carp __PACKAGE__ . "::new: passing 'proxy' in the 'rest_client_config' hash is deprecated. Please, use the corresponding argument instead.\n";
+        $args{proxy} = $proxy unless defined $args{proxy};
+    }
 
-    my $rest = REST::Client->new($rest_client_config);
-
-    # Set proxy to be used
-    $rest->getUseragent->proxy(['http','https'] => $proxy) if $proxy;
+    my $rest = REST::Client->new($args{rest_client_config});
 
     # Set default base URL
-    $rest->setHost($URL);
+    $rest->setHost($args{url});
 
     # Follow redirects/authentication by default
     $rest->setFollow(1);
 
-    # Since JIRA doesn't send an authentication chalenge, we may
-    # simply force the sending of the authentication header.
-    $rest->addHeader(Authorization => 'Basic ' . encode_base64("$username:$password"));
+    # Since JIRA doesn't send an authentication challenge, we force the
+    # sending of the authentication header.
+    $rest->addHeader(Authorization => 'Basic ' . encode_base64("$args{username}:$args{password}"))
+        unless $args{anonymous};
 
-    # Configure UserAgent name
-    $rest->getUseragent->agent(__PACKAGE__);
+    for my $ua ($rest->getUseragent) {
+        # Configure UserAgent name
+        $ua->agent(__PACKAGE__);
+
+        # Set proxy to be used
+        $ua->proxy(['http','https'] => $args{proxy}) if $args{proxy};
+
+        # Turn off SSL verification if requested
+        $ua->ssl_opts(SSL_verify_mode => 0, verify_hostname => 0) if $args{ssl_verify_none};
+    }
 
     return bless {
         rest => $rest,
         json => JSON->new->utf8->allow_nonref,
-        path => $path,
+        api  => $api,
     } => $class;
 }
 
@@ -207,10 +231,11 @@ sub _content {
 sub _build_path {
     my ($self, $path, $query) = @_;
 
-    $path = $self->{path} . $path unless $path =~ m:^/rest/:;
+    # Prefix $path with the default API prefix unless it already specifies one
+    $path = $self->{api} . $path unless $path =~ m:^/rest/:;
 
     if (defined $query) {
-        croak $self->_error("The QUERY argument must be a hash-ref.")
+        croak $self->_error("The QUERY argument must be a hash reference.")
             unless ref $query && ref $query eq 'HASH';
         return $path . '?'. join('&', map {$_ . '=' . uri_escape($query->{$_})} keys %$query);
     } else {
@@ -319,7 +344,7 @@ sub attach_files {
     # FIXME: How to attach all files at once?
     foreach my $file (@files) {
         my $response = $rest->getUseragent()->post(
-            $rest->getHost . "/issue/$issueIdOrKey/attachments",
+            $rest->getHost . "/rest/api/latest/issue/$issueIdOrKey/attachments",
             %{$rest->{_headers}},
             'X-Atlassian-Token' => 'nocheck',
             'Content-Type'      => 'form-data',
@@ -345,13 +370,17 @@ JIRA::REST - Thin wrapper around JIRA's REST API
 
 =head1 VERSION
 
-version 0.015
+version 0.017
 
 =head1 SYNOPSIS
 
     use JIRA::REST;
 
-    my $jira = JIRA::REST->new('https://jira.example.net', 'myuser', 'mypass');
+    my $jira = JIRA::REST->new({
+        URL      => 'https://jira.example.net',
+        username => 'myuser',
+        password => 'mypass',
+    });
 
     # File a bug
     my $issue = $jira->POST('/issue', undef, {
@@ -423,60 +452,91 @@ endpoints have a path prefix of C</rest/agile/VERSION>.
 
 =head1 CONSTRUCTOR
 
-=head2 new URL, USERNAME, PASSWORD [, REST_CLIENT_CONFIG]
+=head2 new HASHREF
 
-The constructor needs up to four arguments:
+=head2 new URL, USERNAME, PASSWORD, REST_CLIENT_CONFIG, ANONYMOUS, PROXY, SSL_VERIFY_NONE
 
-=over
+The constructor can take its arguments from a single hash reference or from
+a list of positional parameters. The first form is preferred because it lets
+you specify only the arguments you need. The second form forces you to pass
+undefined values if you need to pass a specific value to an argument further
+to the right.
 
-=item * URL
+The arguments are described below with the names which must be used as the
+hash keys:
 
-A string or a URI object denoting the base URL of the JIRA
-server. This is a required argument.
+=over 4
+
+=item * B<url>
+
+A string or a URI object denoting the base URL of the JIRA server. This is a
+required argument.
 
 The REST methods described below all accept as a first argument the
 endpoint's path of the specific API method to call. In general you can pass
 the complete path, beginning with the prefix denoting the particular API to
 use (C</rest/api/VERSION>, C</rest/servicedeskapi>, or
-C</rest/agile/VERSION>). However, to make it easier to invoke JIRA's Core
-API if you pass a path not starting with C</rest/> it will be prefixed with
-C</rest/api/latest> or with this URL's path if it has one. This way you can
-choose a specific version of the JIRA Core API to use instead of the latest
-one. For example:
+C</rest/agile/VERSION>). However, you may specify a default API prefix by
+suffixing the URL with it. For example:
 
-    my $jira = JIRA::REST->new('https://jira.example.net/rest/api/1', 'myuser', 'mypass');
+    my $jira = JIRA::REST->new({
+        url      => 'https://jira.example.net/jira/rest/api/1',
+        username => 'myuser',
+        password => 'mypass'
+    });
 
-=item * USERNAME
+    $jira->GET('/rest/api/1/issue/TST-1');
+    $jira->GET('/issue/TST-1');
 
-The username of a JIRA user.
+With this constructor call both GET methods are the same, because the second
+one does not specify an API prefix. This is useful if you mainly want to use
+a particular API or if you want to specify a particular version of an API
+during construction.
 
-It can be undefined if PASSWORD is also undefined. In such a case the
-user credentials are looked up in the C<.netrc> file or via
+=item * B<username>
+
+=item * B<password>
+
+The username and password of a JIRA user to use for authentication.
+
+If B<anonymous> is false then, if either B<username> or B<password> isn't
+defined the module looks them up in either the C<.netrc> file or via
 L<Config::Identity> (which allows C<gpg> encrypted credentials).
 
 L<Config::Identity> will look for F<~/.jira-identity> or F<~/.jira>.
 You can change the filename stub from C<jira> to a custom stub with the
 C<JIRA_REST_IDENTITY> environment variable.
 
-=item * PASSWORD
+=item * B<rest_client_config>
 
-The HTTP password of the user. (This is the password the user uses to
-log in to JIRA's web interface.)
-
-It can be undefined, in which case the user credentials are looked up
-in the C<.netrc> file or via L<Config::Identity>.
-
-=item * REST_CLIENT_CONFIG
-
-A JIRA::REST object uses a REST::Client object to make the REST
-invocations. This optional argument must be a hash-ref that can be fed
-to the REST::Client constructor. Note that the C<URL> argument
+A JIRA::REST object uses a L<REST::Client> object to make the REST
+invocations. This optional argument must be a hash reference that can be fed
+to the REST::Client constructor. Note that the C<url> argument
 overwrites any value associated with the C<host> key in this hash.
 
-To use a network proxy please set the 'proxy' argument to the string or URI
-object describing the fully qualified (including port) URL to your network
-proxy. This is an extension to the REST::Client configuration and will be
-removed from the hash before passing it on to the REST::Client constructor.
+As an extension, the hash reference also accepts one additional argument
+called B<proxy> that is an extension to the REST::Client configuration and
+will be removed from the hash before passing it on to the REST::Client
+constructor. However, this argument is deprecated since v0.017 and you
+should avoid it. Instead, use the following argument instead.
+
+=item * B<proxy>
+
+To use a network proxy set this argument to the string or URI object
+describing the fully qualified URL (including port) to your network proxy.
+
+=item * B<ssl_verify_none>
+
+Sets the C<SSL_verify_mode> and C<verify_hostname ssl> options on the
+underlying L<REST::Client>'s user agent to 0, thus disabling them. This
+allows access to JIRA servers that have self-signed certificates that don't
+pass L<LWP::UserAgent>'s verification methods.
+
+=item * B<anonymous>
+
+Tells the module that you want to connect to the specified JIRA server with
+no username or password.  This way you can access public JIRA servers
+without needing to authenticate.
 
 =back
 
@@ -499,7 +559,7 @@ fields, you pass C</rest/api/latest/field>, and in order to get SLA
 information about an issue you pass
 C</rest/servicedeskapi/request/$key/sla>.
 
-If you're using a method form JIRA Core REST API you may ommit the prefix
+If you're using a method form JIRA Core REST API you may omit the prefix
 C</rest/api/VERSION>. For example, to GET the list of all fields you may
 pass just C</field>.
 
@@ -527,7 +587,7 @@ The PUT and POST methods accept two more arguments:
 =item * VALUE
 
 This is the "entity" being PUT or POSTed. It can be any value, but
-usually is a hash-ref. The value is encoded as a
+usually is a hash reference. The value is encoded as a
 L<JSON|http://www.json.org/> string using the C<JSON::encode> method
 and sent with a Content-Type of C<application/json>.
 
@@ -554,7 +614,7 @@ according to its content type as follows:
 
 The majority of the API's resources return JSON values. Those are
 decoded using the C<decode> method of a C<JSON> object. Most of the
-endpoints return hashes, which are returned as a Perl hash-ref.
+endpoints return hashes, which are returned as a Perl hash reference.
 
 =item * text/plain
 
@@ -600,8 +660,8 @@ This module provides a few utility methods.
 
 =head2 B<set_search_iterator> PARAMS
 
-Sets up an iterator for the search specified by the hash-ref PARAMS. It must
-be called before calls to B<next_issue>.
+Sets up an iterator for the search specified by the hash reference PARAMS.
+It must be called before calls to B<next_issue>.
 
 PARAMS must conform with the query parameters allowed for the
 C</rest/api/2/search> JIRA REST endpoint.
@@ -651,7 +711,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2017 by CPqD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
